<?php
/**
The MIT License

Copyright © 2021-2022 Martin Härtlein

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
**/
// ERROR REPORTING
error_reporting(-1);
ini_set('display_errors', 'On');
set_time_limit(120); 
// CONFIG //////////////////////////////
$imgsizes=[64, 128, 256, 512, 1080, 1920];
$imgsizes_gif=[256];

function rCleanExtension($extensionstr){
	$ext=strtolower($extensionstr);
	if($ext==='jpeg'){
		$ext='jpg';
	}
	return $ext;
}

// MAKE IMAGE (returns id)/////////////////////////////////////////////////////////////
function getOptimizedImageSrc($srcurl, $returnsizeint=0){  
	$pathinfo=pathinfo($srcurl);
	$dirname=$pathinfo['dirname'];
    $filename=$pathinfo['filename'];
    $extension=rCleanExtension($pathinfo['extension']);
	// return url
	$returnurl='';
	$sizepostfix='';
	if($returnsizeint!==0){
		$sizepostfix='_'.$returnsizeint;
	}
	if($extension=='jpg'){
    	$returnurl=$dirname.'/'.$filename.$sizepostfix.'.jpg';
    }
    if($extension=='png'){
     	$returnurl=$dirname.'/'.$filename.$sizepostfix.'.svg';
    }
    if($extension=='gif'){
    	$returnurl=$dirname.'/'.$filename.$sizepostfix.'.gif';
    }
    // check last modified
    $lastresizedlastmodified=456;
    $lastmodified=123;
	
	$checkfile=$dirname.'/lastresizedlastmodified_'.$filename.'.txt';
    if(file_exists($checkfile)){
        $lastresizedlastmodified=file_get_contents($checkfile).'';
    }
	
    if(file_exists($srcurl)){
        $lastmodified=filemtime($srcurl).'';
    }else{
		return 'notfound:'.$srcurl;
	}
    if($lastresizedlastmodified!==$lastmodified || !file_exists($returnurl)){
        if($extension==='jpg'){
        	makeJPGs($srcurl);
		}
        if($extension==='png'){
            makeFFPNG($srcurl);
        }
        if($extension==='gif'){
            makeGIF($srcurl);
        }
    }
	return $returnurl;
}
// MAKE JPG /////////////////////////////////////////////////////////////////////
function makeJPGs($srcurl){
    global $imgsizes;
   	$pathinfo=pathinfo($srcurl);
	$dirname=$pathinfo['dirname'];
	$filename=$pathinfo['filename'];
    $extension=rCleanExtension($pathinfo['extension']);
    // Open the original image
    $oimg=new Imagick();
    $oimg->readImage($srcurl);
    $w=$oimg->getImageWidth();
    $h=$oimg->getImageHeight();
    foreach($imgsizes as $size){
        $resizecopy= clone $oimg;
        $resizecopy->resizeImage($size, $size, Imagick::FILTER_LANCZOS, 1, TRUE);
        $cw=$resizecopy->getImageWidth();
        $ch=$resizecopy->getImageHeight();
        $resizecopy->setImageFormat('jpg');
        $resizecopy->setImageCompression(imagick::COMPRESSION_JPEG);
        $resizecopy->setImageCompressionQuality(72);
        $resizecopy->stripImage();    
        $resizecopy->writeImage($dirname.'/'.$filename.'_'.$size.'.jpg');
        $img_base64src='data:image/jpeg;base64,'.base64_encode($resizecopy->getimageblob());
        $resizecopy->destroy();
    }
    file_put_contents($dirname.'/lastresizedlastmodified_'.$filename.'.txt', filemtime($srcurl).'');
    // destroy all
    $oimg->destroy();
}
// MAKE GIF /////////////////////////////////////////////////////////////////////////
function makeGIF($srcurl){
    global $imgsizes_gif;
   	$pathinfo=pathinfo($srcurl);
	$dirname=$pathinfo['dirname'];
	$filename=$pathinfo['filename'];
    $extension=rCleanExtension($pathinfo['extension']);
    // Open the original image
    $oimg=new Imagick();
    $oimg->readImage($srcurl);
    $w=$oimg->getImageWidth();
    $h=$oimg->getImageHeight();
    foreach($imgsizes_gif as $size){
        // resize
        $resizecopy=clone $oimg;
        $resizecopy=$resizecopy->coalesceImages();
        $wh=getContainWH($w, $h, $size);
        foreach ($resizecopy as $frame) {
          $frame->thumbnailImage($wh[0], $wh[1]);
          $frame->setImagePage($wh[0], $wh[1], 0, 0);
        }
        $cw=$resizecopy->getImageWidth();
        $ch=$resizecopy->getImageHeight();
        $resizecopy=$resizecopy->deconstructImages();
         // save
        $resizecopy->writeImages($dirname.'/'.$filename.'_'.$size.'.gif', true);
        $resizecopy->destroy();
    }
    file_put_contents($dirname.'/lastresizedlastmodified_'.$filename.'.txt', filemtime($srcurl).'');
    // destroy all
    $oimg->destroy();
    
   
}
function getContainWH($ow, $oh, $size){
    $aspect=$ow/$oh;
    $w=$size;
    $h=$w/$aspect;
    if($h>$size){
        $h=$size;
        $w=$h*$aspect;
    }
    return [$w, $h];
}
// MAKE FFPNG ///////////////////////////////////////////////////////////////////////
function makeFFPNG($srcurl){
    global $imgsizes;
    $pathinfo=pathinfo($srcurl);
	$dirname=$pathinfo['dirname'];
	$filename=$pathinfo['filename'];
    $extension=rCleanExtension($pathinfo['extension']);
    // Open the original image
    $oimg = new Imagick();
    $oimg->readImage($srcurl);
    $w=$oimg->getImageWidth();
    $h=$oimg->getImageHeight();
    // make white background jpg
    $img=new Imagick();
    $img->newImage($w, $h, 'white');
    $img->compositeimage($oimg, Imagick::COMPOSITE_OVER, 0, 0);
    // make mask
    $oimg->setImageAlphaChannel(Imagick::ALPHACHANNEL_EXTRACT);
    $oimg->setImageBackgroundColor('white');
    $oimg->setImageAlphaChannel(Imagick::ALPHACHANNEL_SHAPE);
    $mask=new Imagick();
    $mask->newImage($w, $h, 'black');
    $mask->compositeimage($oimg, Imagick::COMPOSITE_OVER, 0, 0);
    foreach($imgsizes as $size){
        // img
        $resizecopy= clone $img;
        $resizecopy->resizeImage($size, $size, Imagick::FILTER_LANCZOS, 1, TRUE);
        $resizecopy->setImageFormat('jpg');
        $resizecopy->setImageCompression(imagick::COMPRESSION_JPEG);
        $resizecopy->setImageCompressionQuality(72);
        $resizecopy->stripImage();
        $cw=$resizecopy->getImageWidth();
        $ch=$resizecopy->getImageHeight();
        $img_base64src='data:image/jpeg;base64,'.base64_encode($resizecopy->getimageblob());
        $resizecopy->destroy();
        // mask
        $resizecopy= clone $mask;
        $resizecopy->resizeImage($size, $size, Imagick::FILTER_LANCZOS, 1, TRUE);
        $resizecopy->setImageFormat('jpg');
        $resizecopy->stripImage();
        $mask_base64src='data:image/jpeg;base64,'.base64_encode($resizecopy->getimageblob()); 
        $resizecopy->destroy();
        // svg
        file_put_contents($dirname.'/'.$filename.'_'.$size.'.svg', rFFPNG_SVG($cw, $ch, $img_base64src, $mask_base64src));
    }
    file_put_contents($dirname.'/lastresizedlastmodified_'.$filename.'.txt', filemtime($srcurl).'');
    // destroy all
    $oimg->destroy();
    $img->destroy();
    $mask->destroy();
}
function rFFPNG_SVG($w, $h, $imgsrc, $masksrc){
    $maskid=createMaskID();
    return '<svg version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" width="'.$w.'px" height="'.$h.'px" viewBox="0 0 '.$w.' '.$h.'" style="enable-background:new 0 0 '.$w.' '.$h.';" xml:space="preserve">
            <defs>
                <mask id="'.$maskid.'" maskUnits="userSpaceOnUse" maskContentUnits="userSpaceOnUse">
                    <image style="overflow:visible;enable-background:new 0 0 '.$w.' '.$h.';" width="'.$w.'" height="'.$h.'" xlink:href="'.$masksrc.'">
                    </image>
                </mask>
            </defs>
            <image style="mask:url(#'.$maskid.');overflow:visible;" width="'.$w.'" height="'.$h.'" xlink:href="'.$imgsrc.'">
            </image>
            </svg>';
}    
function createMaskID(){
    $t=microtime(true);
    $micro=sprintf("%06d", ($t-floor($t))*1000000);
    $d=new DateTime(date('Y-m-d H:i:s.'.$micro, $t));
    $id=$d->format("YmdHisu");
    usleep(2);
    return 'mask_'.$id; 
}  

?>