<?php
/**
The MIT License

Copyright © 2021-2022 Martin Härtlein

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
**/
// GLOBAL TMPID ARRAY FOR STORING USED IDS TEMPORARILY
global $TMPIDS;
$TMPIDS=[];

// WRAP WORDS IN STRING WITH DIVS
function rDivByWordStr($str){
	$arr=explode(' ', $str);
	$rstr='';
	foreach($arr as $word){
		$rstr.='<div>'.$word.'</div>';
	}
	return $rstr;
}

// HREF TO TARGET
function href2target($href){
	if($href.substr($href, 0, 4)==='http'){
		return '_blank'; 
	}
	return '_self';
}

// YOUTUBE THUMBNAIL
function rYtThumbnail($ytid, $quality='medium'){
	if($quality==='small'){
		return 'https://img.youtube.com/vi/'.$ytid.'/sddefault.jpg';
	}
	if($quality==='medium'){
		return 'https://img.youtube.com/vi/'.$ytid.'/mqdefault.jpg';
	}
	if($quality==='high'){
		return 'https://img.youtube.com/vi/'.$ytid.'/hqdefault.jpg';
	}
	if($quality==='max'){
		return 'https://img.youtube.com/vi/'.$ytid.'/maxresdefault.jpg';
	}
}

// ICON FOLDER TO STYLE TAG
function svgFolder2HTMLStyleTag($folderpath){
	$filepaths=glob($folderpath.'/*.svg');
	$stylestr='@charset "utf-8";'.PHP_EOL.':root{'.PHP_EOL;
    foreach($filepaths as $filepath){
        $stylestr.=svg2ccsvar($filepath).PHP_EOL;
    }
	$stylestr.='}';
	return '<style>'.PHP_EOL.$stylestr.PHP_EOL.'</style>'.PHP_EOL;
	
}

// SVG TO CSS VARIABLE 
function svg2ccsvar($url) {
    $str=file_get_contents($url);
    $str=preg_replace('/\v(?:[\v\h]+)/', ' ', $str);
    $str=str_replace('"', "'", $str);
    $str=rawurlencode($str);
	$varname=str_replace('_', '-', basename($url, '.svg'));
    return '--'.$varname.': url("data:image/svg+xml,'.$str.'");';
}

// OG META IMAGE
function rMetaImg($reltobaseurl){
	$imginfo=getimagesize($reltobaseurl);
	if($imginfo!==false){
		return '<meta property="og:image" content="'.ROOTPATH.$reltobaseurl.'">
				<meta property="og:image:width" content="'.$imginfo[0].'">
				<meta property="og:image:height" content="'.$imginfo[1].'">';
	}
	return '';
}

// SVG ICON
function rSVGICON($iconfilename){
	$cssiconurl='url('.HTMLRELPATH.'core/assets/icons/'.$iconfilename.')';
	return '<div class="svgicon" style="mask-image:'.$cssiconurl.';-webkit-mask-image:'.$cssiconurl.';"></div>';
}

// LOGIN
function LOGIN_CHECK(){
	include_once('core/accounts/accounts.php');
	AUTO_INCLUDE_PHP_CSS_JS_byFolder('core/components/login/');
	global $LOGGEDIN;
	global $BACKEND_USER;
	global $USER_OBJ;
	global $USER_ROLE;
	global $USER_NAME;
	global $USER_ID;
	global $URLPARTS;
	if(isset($_SESSION[LOGIN_SESSION_ID])) {
		$USER_OBJ=$BACKEND_USER[$_SESSION[LOGIN_SESSION_ID]];
		if($USER_OBJ!=null){
			$LOGGEDIN=true;
			$USER_ROLE=$USER_OBJ['role'];
			$USER_NAME=$USER_OBJ['username'];
			$USER_ID=$USER_OBJ['userid'];
		}else{
			die('ungültige Session!');
		}
	}
	if($URLPARTS[count($URLPARTS)-1]==='logout' || isset($_GET['logout'])){
		unset($_SESSION[LOGIN_SESSION_ID]); 
		header('Location: '.ROOTPATH.$URLPARTS[0]);
		exit();
	}
}
function ADD_BACKEND_LOGIN($username, $pw, $role){
	global $BACKEND_USER;
	$userid=generateUSERID($username);
	$BACKEND_USER[$userid]['userid']=$userid;
	$BACKEND_USER[$userid]['username']=$username;
	$BACKEND_USER[$userid]['pw']=$pw;
	$BACKEND_USER[$userid]['role']=$role;
}

// DATE VARS
define('DATESTRINGS',
	  json2obj('{	
	  				"monthnames" : {
						"de" : ["Januar", "Februar", "März", "April", "Mai", "Juni", "Juli", "August", "September", "Oktober", "November", "Dezember"],
						"en" : ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"]
					},
	  				"monthnames_short" : {
						"de" : ["Jan", "Feb", "Mär", "Apr", "Mai", "Jun", "Jul", "Aug", "Sep", "Okt", "Nov", "Dez"],
						"en" : ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"]
					},
					"weekdaynames" : {
						"de" : ["Sonntag", "Montag", "Dienstag", "Mittwoch", "Donnerstag", "Freitag", "Samstag"],
						"en" : ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"]
					},
					"weekdaynames_short" : {
						"de" : ["So", "Mo", "Di", "Mi", "Do", "Fr", "Sa"],
						"en" : ["Su", "Mo", "Tu", "We", "Th", "Fr", "Sa"]
					}
	  }')
);
function str2Date($str, $format='long', $lang='de'){
	$timestamp=strtotime($str);
	if($format='long'){
		$weekdayname=DATESTRINGS['weekdaynames'][$lang][date('w', $timestamp)];
		$monthname=DATESTRINGS['monthnames'][$lang][intval(date('n', $timestamp))-1];
		return $weekdayname.', '.date('j', $timestamp).'. '.$monthname.' '.date('Y', $timestamp);
	}else if($format==='short'){
		return date('m.d.Y', $timestamp);
	}
	
}

// OBJ
function rObjValue($obj, $indexstr, $default=''){
	if($obj!==null && isset($obj[$indexstr])){
		return $obj[$indexstr];
	}
	return $default;
}

// CSV
function csv2arr($cvspath, $trimcells=true){
	$csvarr=[];
	$row=0;
	if(($handle=fopen($cvspath, 'r'))!==false) {
		while(($columns=fgetcsv($handle, 1000, ';'))!==false) {
			$csvarr[$row]=[];
			foreach($columns as $cell){
				if($trimcells===true){
					$cell=trim($cell);
				}
				$csvarr[$row][]=removeUTF8BOM($cell);
			}
			$row++;
		}
		fclose($handle);
	}else{
		return null;
	}
	return $csvarr;
}
function cellIsEmpty($csvarr, $row, $col){
	return trim($csvarr[$row][$col])==='';
}
function removeUTF8BOM($str){
    $bom=pack('H*','EFBBBF');
    $str=preg_replace("/^$bom/", '', $str);
    return $str;
}

// AUTO INCLUDE PHP CSS JS
function AUTO_INCLUDE_PHP_CSS_JS_byFolders($rootpath, $addphptodomstrbody=false){
	$folderpaths=glob($rootpath.'*', GLOB_ONLYDIR);
	foreach($folderpaths as $folderpath){
       AUTO_INCLUDE_PHP_CSS_JS_byFolder($folderpath, $addphptodomstrbody);
	}
}
function AUTO_INCLUDE_PHPFUNCTIONS_byFolder($folderpath){
	foreach($GLOBALS as $key=>$val){ 
		global $$key; 
	}
	if(substr($folderpath, -1)!=='/'){
		$folderpath.='/';
	}
    $filepaths=glob($folderpath.'*.php');
    foreach($filepaths as $path){
        if(substr(basename($path), 0, 1)!=='_'){
             ob_start();
             include($path);
			 ob_clean();
        }
    }
}
function AUTO_INCLUDE_PHP_CSS_JS_byFolder($folderpath, $addphptodomstrbody=false){
	foreach($GLOBALS as $key=>$val){ 
		global $$key; 
	}
	$filepaths=glob($folderpath.'*.js');
    foreach($filepaths as $path){
        if(substr(basename($path), 0, 1)!=='_'){
            addHTMLScriptTag($path);
        }
    }
    $filepaths=glob($folderpath.'*.css');
    foreach($filepaths as $path){
        if(substr(basename($path), 0, 1)!=='_'){
            addHTMLStyleTag($path);
        }
    }
    $filepaths=glob($folderpath.'*.php');
    foreach($filepaths as $path){
        if(substr(basename($path), 0, 1)!=='_'){
            if($addphptodomstrbody){
                ob_start();
                include($path);
                $DOMSTR_BODY.=ob_get_clean();
            }else{
                include_once($path);
            }
        }
    }
}
// TAG ELEMENTS
function rHTMLScriptTag($srcpath){
	return '<script type="text/javascript" src="'.HTMLRELPATH.$srcpath.'?v='.filemtime($srcpath).'"></script>'.PHP_EOL;
}
function rHTMLScriptTagCode($codestr, $attstr=''){
	return '<script '.$attstr.'>'.$codestr.'</script>'.PHP_EOL;
}
function rHTMLStyleTag($srcpath){
	return '<link href="'.HTMLRELPATH.$srcpath.'?v='.filemtime($srcpath).'" rel="stylesheet" type="text/css">'.PHP_EOL;
}
function addHTMLScriptTag($srcpath){
	global $DOMSTR_SCRIPTTAGS;
	$DOMSTR_SCRIPTTAGS.=rHTMLScriptTag($srcpath);
}
function addHTMLScriptTagCode($codestr, $attstr=''){
	global $DOMSTR_SCRIPTTAGS;
	$DOMSTR_SCRIPTTAGS.=rHTMLScriptTagCode($codestr, $attstr);
}
function addHTMLScriptTagCodeBefore($codestr, $attstr=''){
	global $DOMSTR_SCRIPTTAGS;
	$DOMSTR_SCRIPTTAGS=rHTMLScriptTagCode($codestr, $attstr).$DOMSTR_SCRIPTTAGS;
}
function addHTMLStyleTag($srcpath){
	global $DOMSTR_STYLETAGS;
	$DOMSTR_STYLETAGS.=rHTMLStyleTag($srcpath);
}

// JSON
function json2obj($json){
    return json_decode($json, true);
}
function obj2json($obj){
    return json_encode((array) $obj); 
}
function getJSONOBJ($url){
	if(!file_exists($url)){
		return null;
	}
	return json2obj(file_get_contents($url));
}
function jsonStr($domstr){
    return trim(json_encode($domstr, JSON_UNESCAPED_UNICODE),'"');
}
function editJSONOBJ($path, $pvarr){
	$o=json2obj(file_get_contents($path));
	foreach($pvarr as $prop=>$value){
		$o[$prop]=$value;
	}
	saveJSON($path, obj2json($o));
}

// SAVE JSON
function doEcho($str){
    echo($str);
}
function saveJSON($url, $json){
    $db_retries_duration=3;
    $db_max_retries=10;
    // new one
    if(!file_exists($url)){
        $folder=pathinfo($url, PATHINFO_DIRNAME);
        if(!file_exists($folder)){
            mkdir($folder, 0777, true);
        }
        $newjson=$json;
        $file=fopen($url, "w") or doEcho("ERROR: UTILS:saveJSON Failed to open $url!");
        flock($file, LOCK_EX);
        fwrite($file, $newjson) or doEcho("ERROR: UTILS:saveJSON Failed to write values to $url!"); 
        flock($file, LOCK_UN);   
        fclose($file);
    // already exists
    }else{
         // try to open file and lock it for writing
        $handle_url=$url;
        $handle=fopen($handle_url,"c+");
        $flocked=flock($handle, LOCK_EX);
        $tries=0;
        while($handle===false || $flocked===false){
            // too much tries -> abort
            if($tries>$db_max_retries){
                echoCmdMessage("error", "Fehler beim Öffnen $url!");
                break;
                return false;
                die();
            }else{
                // lock gescheitert
                sleep($db_retries_duration);
                // try again
                $handle=fopen($handle_url,"c+");
                $flocked=flock($handle, LOCK_EX);
                $tries++;
            }
        }
        $file=fread($handle, filesize($handle_url));
        $oldobj=json_decode($file, true);
        $newjson=$json;
        // write file and unlock
		ftruncate($handle, 0);   
		rewind($handle);        
		fwrite($handle, $newjson);    
		flock($handle, LOCK_UN);   
		fclose($handle);
    }
    return $newjson;
}

// DELETE FOLDER
function deleteDir($dirname) {
	if(is_dir($dirname)){
		$dir_handle=opendir($dirname);
	}
    if(!$dir_handle){
		return false;
	}
	while($file=readdir($dir_handle)) {
     	if($file!="." && $file!=".."){
			if(!is_dir($dirname."/".$file)){
            	unlink($dirname."/".$file);
			}else{
				deleteDir($dirname.'/'.$file);
			}
		}
     }
     closedir($dir_handle);
     rmdir($dirname);
     return true;
}

// STRING FUNCTIONS
function rAttStr($str){
	return str_replace(['"', '&'], ['&quot;', '&amp;'], $str);
}
function rPHPStr($src, $default=''){
	$rstr=$default;
	if(file_exists($src)){
		ob_start();
		include($src);
		$rstr=ob_get_clean();
		if($rstr===''){
			$rstr=$default;
		}
	}
	return $rstr;
}
function replaceNBSP($str){
	$str=str_replace('&nbsp;', ' ', $str);
    $entities = str_replace('&nbsp;', ' ', htmlentities($str));
    return html_entity_decode($entities);
}
function rCleanStr($str){
    return preg_replace('/[^\w]/', '', str_replace(['ä','ö','ü','ß',' '],['ae','oe','ue','ss','_'], strtolower($str)));
}
function rSortStr($str){
	return preg_replace('/[^\w]/', '', str_replace(['ä','ö','ü','ß',' '],['a','o','u','s',''], strtolower($str)));
}
function rSearchStr($str){
	$str=rFilenameStr($str);
	return preg_replace('/[^\w]/', '', str_replace(['ä','ö','ü','ß',' '],['a','o','u','s',' '], strtolower($str)));
}
function generatePWHASH($pw){
	return password_hash($pw, PASSWORD_DEFAULT);
}
function generateUSERID($mail){
	$sha1=sha1(strtolower($mail));
	return substr($sha1, 0, 10).'/'.substr($sha1, 10, 10).'/'.substr($sha1, 20, 10).'/'.substr($sha1, 30, 10).'/';
}
function rID(){
    // e.g. 2019/08/16/19/29/48586003/
    $t=microtime(true);
    $micro=sprintf("%06d", ($t-floor($t))*1000000);
    $d=new DateTime(date('Y-m-d H:i:s.'.$micro, $t));
    $id=$d->format("Y-m-d H:i:s.u");
    $id=str_replace(['-',' ',':'], '/', $id);
    $id=str_replace('.', '', $id).'/';
    usleep(23);
    return $id; 
}
function cleanID2ID($cleanid){
    $microintstr=base_convert($cleanid, 36, 10);
    $microtime=$microintstr/1000000;
    $d=new DateTime(date('Y-m-d H:i:s', $microtime));
    return $d->format("Y/m/d/H/i/s").substr($microintstr, -6).'/';
}
function ID2cleanID($id){
    $int=str_replace('/', '', $id);
    $Y=substr($int, 0, 4);
    $m=substr($int, 4, 2);
    $d=substr($int, 6, 2);
    $H=substr($int, 8, 2);
    $i=substr($int, 10, 2);
    $s=substr($int, 12, 2);
    $micro=substr($int, 14);
    $dt=new DateTime(date($Y.'-'.$m.'-'.$d.' '.$H.':'.$i.':'.$s.'.'.$micro));
    $microtime=$dt->format('U.u');
    $microintstr=str_replace('.', '', $microtime);
    $cleanid=base_convert($microintstr, 10, 36);
    return $cleanid;
}
function isValidId($id){
	$idarr=ffexplode('/', $id);
	return (strlen($id)===26 && count($idarr)===7 && strlen($idarr[0])===4 && strlen($idarr[1])===2 && strlen($idarr[2])===2 && strlen($idarr[3])===2 && strlen($idarr[4])===2 && strlen($idarr[5])===8);
}
function rTmpID($prefix='tmp_', $varname='LAST'){
	global $TMPIDS;
	$TMPIDS[$varname]=$prefix.rand().str_replace('.','',microtime(true));
    return $TMPIDS[$varname];
}
function rFilenameStr($str){
	$replace=[
		'&lt;' => '', '&gt;' => '', '&#039;' => '', '&amp;' => '',
		'&quot;' => '', 'À' => 'A', 'Á' => 'A', 'Â' => 'A', 'Ã' => 'A', 'Ä' => 'Ae',
		'&Auml;' => 'A', 'Å' => 'A', 'Ā' => 'A', 'Ą' => 'A', 'Ă' => 'A', 'Æ' => 'Ae',
		'Ç' => 'C', 'Ć' => 'C', 'Č' => 'C', 'Ĉ' => 'C', 'Ċ' => 'C', 'Ď' => 'D', 'Đ' => 'D',
		'Ð' => 'D', 'È' => 'E', 'É' => 'E', 'Ê' => 'E', 'Ë' => 'E', 'Ē' => 'E',
		'Ę' => 'E', 'Ě' => 'E', 'Ĕ' => 'E', 'Ė' => 'E', 'Ĝ' => 'G', 'Ğ' => 'G',
		'Ġ' => 'G', 'Ģ' => 'G', 'Ĥ' => 'H', 'Ħ' => 'H', 'Ì' => 'I', 'Í' => 'I',
		'Î' => 'I', 'Ï' => 'I', 'Ī' => 'I', 'Ĩ' => 'I', 'Ĭ' => 'I', 'Į' => 'I',
		'İ' => 'I', 'Ĳ' => 'IJ', 'Ĵ' => 'J', 'Ķ' => 'K', 'Ł' => 'K', 'Ľ' => 'K',
		'Ĺ' => 'K', 'Ļ' => 'K', 'Ŀ' => 'K', 'Ñ' => 'N', 'Ń' => 'N', 'Ň' => 'N',
		'Ņ' => 'N', 'Ŋ' => 'N', 'Ò' => 'O', 'Ó' => 'O', 'Ô' => 'O', 'Õ' => 'O',
		'Ö' => 'Oe', '&Ouml;' => 'Oe', 'Ø' => 'O', 'Ō' => 'O', 'Ő' => 'O', 'Ŏ' => 'O',
		'Œ' => 'OE', 'Ŕ' => 'R', 'Ř' => 'R', 'Ŗ' => 'R', 'Ś' => 'S', 'Š' => 'S',
		'Ş' => 'S', 'Ŝ' => 'S', 'Ș' => 'S', 'Ť' => 'T', 'Ţ' => 'T', 'Ŧ' => 'T',
		'Ț' => 'T', 'Ù' => 'U', 'Ú' => 'U', 'Û' => 'U', 'Ü' => 'Ue', 'Ū' => 'U',
		'&Uuml;' => 'Ue', 'Ů' => 'U', 'Ű' => 'U', 'Ŭ' => 'U', 'Ũ' => 'U', 'Ų' => 'U',
		'Ŵ' => 'W', 'Ý' => 'Y', 'Ŷ' => 'Y', 'Ÿ' => 'Y', 'Ź' => 'Z', 'Ž' => 'Z',
		'Ż' => 'Z', 'Þ' => 'T', 'à' => 'a', 'á' => 'a', 'â' => 'a', 'ã' => 'a',
		'ä' => 'ae', '&auml;' => 'ae', 'å' => 'a', 'ā' => 'a', 'ą' => 'a', 'ă' => 'a',
		'æ' => 'ae', 'ç' => 'c', 'ć' => 'c', 'č' => 'c', 'ĉ' => 'c', 'ċ' => 'c',
		'ď' => 'd', 'đ' => 'd', 'ð' => 'd', 'è' => 'e', 'é' => 'e', 'ê' => 'e',
		'ë' => 'e', 'ē' => 'e', 'ę' => 'e', 'ě' => 'e', 'ĕ' => 'e', 'ė' => 'e',
		'ƒ' => 'f', 'ĝ' => 'g', 'ğ' => 'g', 'ġ' => 'g', 'ģ' => 'g', 'ĥ' => 'h',
		'ħ' => 'h', 'ì' => 'i', 'í' => 'i', 'î' => 'i', 'ï' => 'i', 'ī' => 'i',
		'ĩ' => 'i', 'ĭ' => 'i', 'į' => 'i', 'ı' => 'i', 'ĳ' => 'ij', 'ĵ' => 'j',
		'ķ' => 'k', 'ĸ' => 'k', 'ł' => 'l', 'ľ' => 'l', 'ĺ' => 'l', 'ļ' => 'l',
		'ŀ' => 'l', 'ñ' => 'n', 'ń' => 'n', 'ň' => 'n', 'ņ' => 'n', 'ŉ' => 'n',
		'ŋ' => 'n', 'ò' => 'o', 'ó' => 'o', 'ô' => 'o', 'õ' => 'o', 'ö' => 'oe',
		'&ouml;' => 'oe', 'ø' => 'o', 'ō' => 'o', 'ő' => 'o', 'ŏ' => 'o', 'œ' => 'oe',
		'ŕ' => 'r', 'ř' => 'r', 'ŗ' => 'r', 'š' => 's', 'ù' => 'u', 'ú' => 'u',
		'û' => 'u', 'ü' => 'ue', 'ū' => 'u', '&uuml;' => 'ue', 'ů' => 'u', 'ű' => 'u',
		'ŭ' => 'u', 'ũ' => 'u', 'ų' => 'u', 'ŵ' => 'w', 'ý' => 'y', 'ÿ' => 'y',
		'ŷ' => 'y', 'ž' => 'z', 'ż' => 'z', 'ź' => 'z', 'þ' => 't', 'ß' => 'ss',
		'ſ' => 'ss', 'ый' => 'iy', 'А' => 'A', 'Б' => 'B', 'В' => 'V', 'Г' => 'G',
		'Д' => 'D', 'Е' => 'E', 'Ё' => 'YO', 'Ж' => 'ZH', 'З' => 'Z', 'И' => 'I',
		'Й' => 'Y', 'К' => 'K', 'Л' => 'L', 'М' => 'M', 'Н' => 'N', 'О' => 'O',
		'П' => 'P', 'Р' => 'R', 'С' => 'S', 'Т' => 'T', 'У' => 'U', 'Ф' => 'F',
		'Х' => 'H', 'Ц' => 'C', 'Ч' => 'CH', 'Ш' => 'SH', 'Щ' => 'SCH', 'Ъ' => '',
		'Ы' => 'Y', 'Ь' => '', 'Э' => 'E', 'Ю' => 'YU', 'Я' => 'YA', 'а' => 'a',
		'б' => 'b', 'в' => 'v', 'г' => 'g', 'д' => 'd', 'е' => 'e', 'ё' => 'yo',
		'ж' => 'zh', 'з' => 'z', 'и' => 'i', 'й' => 'y', 'к' => 'k', 'л' => 'l',
		'м' => 'm', 'н' => 'n', 'о' => 'o', 'п' => 'p', 'р' => 'r', 'с' => 's',
		'т' => 't', 'у' => 'u', 'ф' => 'f', 'х' => 'h', 'ц' => 'c', 'ч' => 'ch',
		'ш' => 'sh', 'щ' => 'sch', 'ъ' => '', 'ы' => 'y', 'ь' => '', 'э' => 'e',
		'ю' => 'yu', 'я' => 'ya', ' ' => '-'
	];
	$str=str_replace(array_keys($replace), $replace, $str);  
    return preg_replace('/[^\w-]/', '', $str);
}
?>