// INPUTS
// dropdowns
let input_current_expanded_dropdown=null;

// helper

function parseBoolean(value){
	value=String(value).toLowerCase();
	return (value==='true' || value==='yes' || value==='1' || value==='ja')
}


(function(){
	// API /////////////////////////////////////////////////////////////////////////////////////////////////
	
	// INPUT CHILDS TO OBJ
	BLUEPRINTS['childInputsToObj']=function(el){
		const obj={};
		const inputs=el.querySelectorAll('*[input-type]');
		for(let input of inputs){
			const name=input.getAttribute('input-name');
			const type=input.getAttribute('input-type');
			obj[name]=BLUEPRINTS['getInputValue'](input);
		}
		return obj;
	}
	// GROUP TO  OBJECT
	BLUEPRINTS['resetInputGroup']=function(group){
		const inputs=group.querySelectorAll('*[input-type]');
		for(let input of inputs){
			BLUEPRINTS['resetInput'](input);
		}
	}
	BLUEPRINTS['inputGroupToObj']=function(group){
		const groupobj={};
		groupobj.type=group.getAttribute('inputgroup-type');
		groupobj.name=group.getAttribute('inputgroup-name');
		groupobj.id=group.getAttribute('inputgroup-id');
		const inputs=group.querySelectorAll('*[input-type]');
		groupobj.inputs=[];
		for(let input of inputs){
			inputobj=BLUEPRINTS['inputToObj'](input);
			groupobj.inputs[inputobj.name]=inputobj;
		}
		return groupobj;
	}
	//  INPUTS TO OBJECT
	BLUEPRINTS['inputToObj']=function(input){
		const type=input.getAttribute('input-type');
		return BLUEPRINTS[type+'_toObj'](input);
	}
	BLUEPRINTS['setInputValue']=function(input, value){
		const type=input.getAttribute('input-type');
		BLUEPRINTS[type+'_setValue'](input, value);
	}
	BLUEPRINTS['getInputValue']=function(input){
		const type=input.getAttribute('input-type');
		return BLUEPRINTS[type+'_getValue'](input);
	} 
	BLUEPRINTS['resetInput']=function(input){
		const type=input.getAttribute('input-type');
		return BLUEPRINTS[type+'_resetValue'](input);
	} 
	
	// INPUT TYPES /////////////////////////////////////////////////////////////////////////////
	
	// DROPDOWN
	BLUEPRINTS['dropdown_toObj']=function(input){
		const inputobj={};
		inputobj.type=input.getAttribute('input-type');
		inputobj.name=input.getAttribute('input-name');
		inputobj.value=input.getAttribute('input-value');
		inputobj.label=input.getAttribute('input-label');
        inputobj.options=[];
        const dropdownitems=input.querySelectorAll('dropdownitem[set-value]');
        for(dropdownitem of dropdownitems){ 
            const ovalue=dropdownitem.setAttribute('set-value');
            const otext=dropdownitem.innerHTML; 
            const optionobj={};
            optionobj.value=ovalue;
            optionobj.text=otext;
            inputobj.options.push(optionobj);
        }
		return inputobj;
	}
	BLUEPRINTS['dropdown_resetValue']=function(input){
		const initialvalue=input.getAttribute('input-initial-value');
		BLUEPRINTS['dropdown_setValue'](input, initialvalue);
	}
	BLUEPRINTS['dropdown_setValue']=function(dropdown, value){
		let dropdownitem=null;
		dropdown.setAttribute('input-value', value);
		const dropdownitems=dropdown.getElementsByClassName('dropdownitem');
		let valuetext='-invalid-value-';
        for(let item of dropdownitems){
			if(item.getAttribute('set-value')===value){
				item.setAttribute('is-current','');
				valuetext=item.innerHTML;
			}else{
				item.removeAttribute('is-current');
			}
        }
        dropdown.getElementsByClassName('currentvalue')[0].innerHTML=valuetext;
        if(dropdown.hasAttribute('onvaluechange-callback')){
            callWindowCallback(dropdown.getAttribute('onvaluechange-callback'), dropdown);
        }
	}
	BLUEPRINTS['dropdown_getValue']=function(dropdown){
		return dropdown.getAttribute('input-value');
	}
	BLUEPRINTS['dropdown_click']=function(e){
		console.log('dropdown_click', e);
		const dropdown=e.currentTarget.closest('div[input-type="dropdown"]');
		const resetbutton=e.target.closest('.resetbutton');
		const dropdownitem=e.target.closest('.dropdownitem');
		// close other if opened
		if(input_current_expanded_dropdown!==dropdown && input_current_expanded_dropdown!==null){
			closeCurrentDropDown();
		}
		// expand
		if(input_current_expanded_dropdown===null && resetbutton===null){
			input_current_expanded_dropdown=dropdown;
			setTimeout(expandCurrentDropDown, 1);
		}
		// on dropdownitem
		if(dropdownitem!==null){
			BLUEPRINTS['dropdown_setValue'](dropdown, dropdownitem.getAttribute('set-value'));
		}
		// on resetbutton
		if(resetbutton!==null){
			BLUEPRINTS['dropdown_setValue'](dropdown, '');
			//BLUEPRINTS['dropdown_resetValue'](dropdown);
		}
	}
	function closeCurrentDropDown(){
		if(input_current_expanded_dropdown!==null){
			input_current_expanded_dropdown.removeAttribute('is-expanded');
			window.removeEventListener('click', closeCurrentDropDown);
			input_current_expanded_dropdown=null;
		}
	}
	function expandCurrentDropDown(){
		if(input_current_expanded_dropdown!==null){
			input_current_expanded_dropdown.setAttribute('is-expanded', '');
			window.removeEventListener('click', closeCurrentDropDown);
			window.addEventListener('click', closeCurrentDropDown);
		}
	}
	
	// TEXTINPUT
	BLUEPRINTS['textinput_toObj']=function(input){
		const inputobj={};
		inputobj.type=input.getAttribute('input-type');
		inputobj.name=input.getAttribute('input-name');
		inputobj.value=input.getAttribute('input-value');
		inputobj.label=input.getAttribute('input-label');
		return inputobj;
	}
	BLUEPRINTS['textinput_resetValue']=function(input){
		const initialvalue=input.getAttribute('input-initial-value');
		BLUEPRINTS['textinput_setValue'](input, initialvalue);
	}
	BLUEPRINTS['textinput_setValue']=function(textinput, value){
		textinput.setAttribute('input-value', value);
		if(textinput.hasAttribute('onvaluechange-callback')){
			callWindowCallback(textinput.getAttribute('onvaluechange-callback'), textinput);
		}
	}
	BLUEPRINTS['textinput_getValue']=function(textinput){
		return textinput.getAttribute('input-value');
	}
 	BLUEPRINTS['textinput_oninput']=function(e){
		const textinput=e.currentTarget.closest('div[input-type="textinput"]');
		BLUEPRINTS['textinput_setValue'](textinput,  e.currentTarget.value);
	}
	
	// RICH TEXT INPUT
	BLUEPRINTS['richtextinput_toObj']=function(input){
		const inputobj={};
		inputobj.type=input.getAttribute('input-type');
		inputobj.name=input.getAttribute('input-name');
		inputobj.value=input.querySelectorAll('div[editor-tags]')[0].value;
		inputobj.label=input.getAttribute('input-label');
		return inputobj;
	}
	BLUEPRINTS['richtextinput_resetValue']=function(input){
		const initialvalue=input.getAttribute('input-initial-value');
		BLUEPRINTS['richtextinput_setValue'](input, initialvalue);
	}
	BLUEPRINTS['richtextinput_setValue']=function(richtextinput, value){
		richtextinput.setAttribute('input-value', value);
		if(richtextinput.hasAttribute('onvaluechange-callback')){
			callWindowCallback(richtextinput.getAttribute('onvaluechange-callback'), richtextinput);
		}
	}
	BLUEPRINTS['richtextinput_getValue']=function(richtextinput){
		return richtextinput.querySelectorAll('div[editor-tags]')[0].innerHTML;
	}
	
 	BLUEPRINTS['richtextinput_oninput']=function(e){
		const richtextinput=e.currentTarget.closest('div[input-type="richtextinput"]');
		TEXTEDITOR.richtextinput_oninput(e);
		if(richtextinput.hasAttribute('onvaluechange-callback')){
			callWindowCallback(richtextinput.getAttribute('onvaluechange-callback'), richtextinput);
		}
	}
	BLUEPRINTS['richtextinput_insertToolbar']=function(targetid, editortags, texteditor_toolbar_id){
		const tmptoolbar=TEXTEDITOR.rToolbar(editortags);
		const target=document.querySelectorAll('*[target-id="'+targetid+'"]')[0];
		tmptoolbar.setAttribute('texteditor-toolbar-id', texteditor_toolbar_id);
		target.insertAdjacentElement('afterend', tmptoolbar);
		REMOVE(target);
	}
	
	// TEXTAREA
	BLUEPRINTS['textarea_toObj']=function(input){
		const inputobj={};
		inputobj.type=input.getAttribute('input-type');
		inputobj.name=input.getAttribute('input-name');
		inputobj.value=input.getElementsByTagName('TEXTAREA')[0].value;
		inputobj.label=input.getAttribute('input-label');
		return inputobj;
	}
	BLUEPRINTS['textarea_resetValue']=function(input){
		const initialvalue=input.getAttribute('input-initial-value');
		BLUEPRINTS['textinput_setValue'](input, initialvalue);
	}
	BLUEPRINTS['textarea_setValue']=function(textarea, value){
		input.getElementsByTagName('TEXTAREA')[0].value=value;
		if(textarea.hasAttribute('onvaluechange-callback')){
			callWindowCallback(textarea.getAttribute('onvaluechange-callback'), textarea);
		}
	}
	BLUEPRINTS['textarea_getValue']=function(textarea){
		return textarea.getElementsByTagName('TEXTAREA')[0].value;
	}
 	BLUEPRINTS['textarea_oninput']=function(e){
		const textarea=e.currentTarget.closest('div[input-type="textarea"]');
		if(textarea.hasAttribute('onvaluechange-callback')){
			callWindowCallback(textarea.getAttribute('onvaluechange-callback'), textarea);
		}
	}
	
	// CHECKBOX
	BLUEPRINTS['checkbox_toObj']=function(input){
		const inputobj={};
		inputobj.type=input.getAttribute('input-type');
		inputobj.name=input.getAttribute('input-name');
		inputobj.value=input.getAttribute('input-value');
		inputobj.label=input.getAttribute('input-label');
		return inputobj;
	}
	BLUEPRINTS['checkbox_setValue']=function(checkbox, value){
		checkbox.setAttribute('input-value', parseBoolean(value));
		if(checkbox.hasAttribute('onvaluechange-callback')){
			callWindowCallback(checkbox.getAttribute('onvaluechange-callback'), checkbox);
		}
	}
	BLUEPRINTS['checkbox_getValue']=function(checkbox){
		return parseBoolean(checkbox.getAttribute('input-value'));
	}		
	BLUEPRINTS['checkbox_resetValue']=function(input){
		const initialvalue=input.getAttribute('input-initial-value');
		BLUEPRINTS['checkbox_setValue'](input, initialvalue);
	}
 	BLUEPRINTS['checkbox_oninput']=function(e){
		const checkbox=e.currentTarget.closest('div[input-type="checkbox"]');
		BLUEPRINTS['checkbox_setValue'](checkbox, !BLUEPRINTS['checkbox_getValue'](checkbox));
	}

	
	// DATE INPUT 
	BLUEPRINTS['dateinput_toObj']=function(input){
		const inputobj={};
		inputobj.type=input.getAttribute('input-type');
		inputobj.name=input.getAttribute('input-name');
		inputobj.value=input.getAttribute('input-value');
		inputobj.label=input.getAttribute('input-label');
		return inputobj;
	}
	BLUEPRINTS['dateinput_resetValue']=function(input){
		const initialvalue=input.getAttribute('input-initial-value');
		BLUEPRINTS['dateinput_setValue'](input, initialvalue);
	}
	BLUEPRINTS['dateinput_setValue']=function(dateinput, value){
		dateinput.setAttribute('input-value', value);
		if(dateinput.hasAttribute('onvaluechange-callback')){
			callWindowCallback(dateinput.getAttribute('onvaluechange-callback'), dateinput);
		}
	}
	BLUEPRINTS['dateinput_getValue']=function(dateinput){
		return dateinput.getAttribute('input-value');
	}
	
 	BLUEPRINTS['dateinput_oninput']=function(e){
		const dateinput=e.currentTarget.closest('div[input-type="dateinput"]');
		BLUEPRINTS['dateinput_setValue'](dateinput,  e.currentTarget.value);
	}
	
	// MEDIA INPUT 
	
	BLUEPRINTS['mediainput_getValue']=function(input){
		const mediaels=input.querySelectorAll('.mediael[media-id]');
		let mediaids=[];
		for(mediael of mediaels){
			mediaids.push(mediael.getAttribute('media-id'));
		}
		return mediaids;
	}
	BLUEPRINTS['mediainput_oninput']=function(e){
		UPLOAD.addFilesByInputEvent(e);
		// BLUEPRINTS['dateinput_setValue'](dateinput,  e.currentTarget.value);
	}

	BLUEPRINTS['mediainput_onclick']=function(e){
        // delete mediael bt
        const deletebt=e.target.closest('.deletebt');
        if(deletebt!==null){
            if(deletebt.hasAttribute('ready-to-delete')){
                const mediael=deletebt.closest('div[media-id][target-id]');
                mediael.style.opacity='0.5';
                const mediaid=mediael.getAttribute('media-id');
                const targetid=mediael.getAttribute('target-id')
                const callobj={};
                callobj['api-func-component']='INPUTS';
                callobj['api-func-name']='mediainput_deleteMedia';
                callobj['api-target-id']=targetid;
                callobj['api-insert-cmd']='replaceself';
                callobj['api-func-parobj']=mediaid;
                API_CALL([callobj]);
                deletebt.removeAttribute('ready-to-delete');
            }else{
                deletebt.setAttribute('ready-to-delete', '');
            }
        }
	}
	
	// GLOBAL
	window.addEventListener('click', INPUTS_ONCLICK);
	function INPUTS_ONCLICK(e){
		// remove ready-to-delete if not on ready-to-delete element
		if(e.target.closest('*[ready-to-delete]')===null){
			const els=document.querySelectorAll('*[ready-to-delete]');
			for(el of els){
				el.removeAttribute('ready-to-delete');
			}				  
		}
		
	}
	
})();
