// JavaScript Document
window.GRIDMAKER={};
(function(){
	document.addEventListener("DOMContentLoaded", INIT);
	function INIT(){
		const canvas=document.createElement('CANVAS');
		let stageToCanvas_firsttime=true;
		const context=canvas.getContext('2d');
		const gridmaker=document.getElementsByClassName('gridmaker')[0];
		const stage=gridmaker.getElementsByClassName('stage')[0];
		const blurlayer0=document.getElementsByClassName('blurlayer0')[0];
		const halloffameformbox=gridmaker.getElementsByClassName('halloffameformbox')[0];
		const grids_layer=stage.getElementsByClassName('grids_layer')[0];
		const broken_layer=stage.getElementsByClassName('broken_layer')[0];
		const fills_layer=stage.getElementsByClassName('fills_layer')[0];
		const fills_svg=fills_layer.querySelectorAll('svg.fills')[0];
		const fills_polygongroup=fills_layer.querySelectorAll('g.fills_polygongroup')[0];
		const ui_layer=stage.getElementsByClassName('ui_layer')[0];
		const colorpalette=document.getElementsByClassName('colorpalette')[0];
		const palettecolors=colorpalette.querySelectorAll('input');
		let hitlas, posindex2la;
		// INIT 
		GRIDMAKER.INIT=function(){
			// set colors to style attribute of fills_polygongroup for 
			for(const palettecolor of palettecolors){
				fills_polygongroup.style.setProperty('--'+palettecolor.getAttribute('id'), palettecolor.value);
			}
			// append canvas to halloffameform
			halloffameformbox.getElementsByClassName('canvaspreview')[0].appendChild(canvas);
			// contain stage
			const fitel=stage;
			const padding=32;
			const parent_rect=fitel.parentElement.getBoundingClientRect();
			const scalef1=(parent_rect.width-padding*2)/stage.offsetWidth;
			const scalef2=(parent_rect.height-padding*2)/stage.offsetHeight
			const scale=Math.min(scalef1, scalef2)*0.5;
			setTransform(fitel, {x:padding, y:padding, scx:scale, scy:scale, rot:0});
			//
			GRIDMAKER.initStage();
			GRIDMAKER.openLibrary();
		}
		
		// INLINE LISTENERS 
		GRIDMAKER.openLibrary=function(){
			gridmaker.setAttribute('library-is-open', '');
		}
		GRIDMAKER.closeLibrary=function(){
			gridmaker.removeAttribute('library-is-open');
		}
		
		// libraryitem on click
		GRIDMAKER.libraryitem_onclick=function(e){
			const svg=e.currentTarget.querySelectorAll('.media>svg')[0];
			GRIDMAKER.addGrid(svg)
			GRIDMAKER.closeLibrary();
		}
		// broken_layer_onclick -> delete line
		GRIDMAKER.broken_layer_onclick=function(e){
			if(e.target.tagName.toLowerCase()==='line'){
				const x1=parseFloat(e.target.getAttribute('x1'));
				const y1=parseFloat(e.target.getAttribute('y1'));
				const x2=parseFloat(e.target.getAttribute('x2'));
				const y2=parseFloat(e.target.getAttribute('y2'));
				// prevent deleting top left bottom and right lines of canvas
				const isborderline=(x1===0 && x2===0) || (y1===0 && y2===0) || (x1===2880 && x2===2880) || (y1===2880 && y2===2880);
				if(!isborderline){
					const posindex1=x1+'_'+y1;
					const posindex2=x2+'_'+y2;
					//  la1
					let n=posindex2la[posindex1].las.length, la;
					while(n--){
						la=posindex2la[posindex1].las[n];
						if(la[2]===x2 && la[3]===y2){
							posindex2la[posindex1].las.splice(n, 1);
						}
					}
					// la2
					n=posindex2la[posindex2].las.length
					while(n--){
						la=posindex2la[posindex2].las[n];
						if(la[2]===x1 && la[3]===y1){
							posindex2la[posindex2].las.splice(n, 1);
						}
					}
					GRIDMAKER.deleteKnotWithNoConnection();
					GRIDMAKER.updatebrokenGrid();
				}
			}
		}
		// beginfillbutton on lcick
		GRIDMAKER.beginfillbutton_onclick=function(){
			GRIDMAKER.setToolMode('fill');
		}
		// palettecolor on input
		GRIDMAKER.palettecolor_oninput=function(e){
			for(const palettecolor of palettecolors){
				palettecolor.removeAttribute('is-active');
			}
			e.currentTarget.setAttribute('value', e.currentTarget.value);
			e.currentTarget.setAttribute('is-active', '');
			fills_polygongroup.style.setProperty('--'+e.currentTarget.getAttribute('id'), e.currentTarget.value);
		}
		GRIDMAKER.palettecolor_onclick=function(e){
			for(const palettecolor of palettecolors){
				palettecolor.removeAttribute('is-active');
			}
			e.currentTarget.setAttribute('is-active', '');
			stage.style.setProperty('--'+e.currentTarget.getAttribute('id'), e.currentTarget.value);
		}
		GRIDMAKER.fills_layer_onclick=function(e){
			// check if has Polygon
			const polygon=e.target.closest('.fillpolygon');
			if(polygon!==null){
				polygon.style.setProperty('fill','var(--'+colorpalette.querySelectorAll('input[is-active]')[0].getAttribute('id')+')');
			// else create Polygon	if has brokengrid
			}else{
				// get nearest line
				const gt=getGlobalTransform(stage);
				const mx=e.clientX;
				const my=e.clientY;
				const px=(mx-gt.pos.x)/gt.scale.x;
				const py=(my-gt.pos.y)/gt.scale.x;
                const nla=hitlas[getNearestLinearrIndex(px, py, hitlas)];
                inverse=!isLeftToLine(px, py, nla);
                // get neighborlines min angle is next till startindexpos
                let polygon_points='';
                let minangle=Number.MAX_VALUE;
                let tmpla, mintmpla, angle, cx , cy, x1, y1;
                startindex=nla[0]+'_'+nla[1]
                currentposindex=startindex;
                currentknot=posindex2la[currentposindex];
                currentla=nla;
                polygon_points=nla[0]+','+nla[1]+' '+nla[2]+','+nla[3]+' ';
                let maxcount=300, count=0;
                function next(){
                    count++;
                    currentposindex=currentla[2]+'_'+currentla[3];
                    if(count<maxcount && currentposindex!==startindex){
                        currentknot=posindex2la[currentposindex];
                        x1=currentla[0];
                        y1=currentla[1];
                        cx=currentla[2];
                        cy=currentla[3];
                        mintmpla=null;
                        i=currentknot.las.length;
                        minangle=Number.MAX_VALUE;
                        while(i--){
                            tmpla=currentknot.las[i];
                            if(!(tmpla[0]===currentla[2] && tmpla[1]===currentla[3] && tmpla[2]===currentla[0] && tmpla[3]===currentla[1])){
                                angle=getAngle(cx, cy, x1, y1, tmpla[2], tmpla[3]);
								console.log(angle);
                                if(inverse){
                                    angle=Math.PI*2-angle;
                                }
                                if(angle<minangle){
                                    minangle=angle;
                                    mintmpla=tmpla;
                                }	
                            }else{
                                //console.log('same gegenvector: tmpla', tmpla, 'currentla:', currentla);
                            }
                        }
                        currentla=mintmpla;
                        polygon_points+=currentla[2]+','+currentla[3]+' ';
                        next();
                    }
                }

                next();
                polygon_points=polygon_points.trim();
                if(xyHitsPolygonpoints(px, py, polygon_points)){
                    const polygon_style='style="fill:var(--'+colorpalette.querySelectorAll('input[is-active]')[0].getAttribute('id')+');"';
                    const polygonstr='<polygon class="fillpolygon" points="'+polygon_points+'" '+polygon_style+' />';
                    fills_polygongroup.insertAdjacentHTML('beforeend', polygonstr);									 
                }else{

                }
					
					
					
				//}
			}
			
		}
		GRIDMAKER.freezegridelbutton_onclick=function(e){
			const grideldraghelpers=grids_layer.querySelectorAll('.grideldraghelper');
			const linearrs=[];
			let n;
			for(const grideldraghelper of grideldraghelpers){
				const gridel_lines=grideldraghelper.querySelectorAll('line');
				const tmp_linearrs=lines2Linearrs(gridel_lines);
				transformLinearrs(tmp_linearrs, grideldraghelper);
				for(n=0; n<tmp_linearrs.length; n++){
					linearrs.push([tmp_linearrs[n][0],tmp_linearrs[n][1], tmp_linearrs[n][2], tmp_linearrs[n][3]]);
				}
			}
			const las=getBrokenLinearrs(linearrs);
			// prepare for fill
           

            // add reveres linearrs start <-> end
			let la;
            n=las.length;
            while(n--){
                la=las[n];
                las.push([la[2], la[3], la[0], la[1]]);
            }

            // build posindex2la  
           	posindex2la={}; 
			let startindex, endindex;
            n=las.length; 
            while(n--){
                la=las[n];
                // add startindex positionindex
                startindex=la[0]+'_'+la[1];
                if(!posindex2la.hasOwnProperty(startindex)){
                    posindex2la[startindex]={las:[]};
                }
                posindex2la[startindex].las.push([la[0], la[1], la[2], la[3]]);
            }
			console.log('posindex2la:', JSON.parse(JSON.stringify(posindex2la)));
            // remove knots with no connections
            GRIDMAKER.deleteKnotWithNoConnection();
			console.log('posindex2la no connections:', JSON.parse(JSON.stringify(posindex2la)));
            // refill hitlas and plot		
			GRIDMAKER.updatebrokenGrid();
			// set Tool Mode to delete 
			GRIDMAKER.setToolMode('delete');
		}
		GRIDMAKER.updatebrokenGrid=function(){
			hitlas=[];
			let domstr='', n, currentknot;
            for(let posindex in posindex2la){
                currentknot=posindex2la[posindex];
                n=currentknot.las.length;
                while(n--){
                    hitlas.push(currentknot.las[n]);
					domstr+=linearr2Linedomstr(currentknot.las[n]);
                }
            }
			console.log('hitlas', hitlas);
			broken_layer.innerHTML=`<svg class="brokengrid" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" viewBox="0 0 2880 2880" width="2880" height="2880">
										<style>
											 svg.brokengrid line {
												  stroke: white;
												  stroke-width: 32px;
											  }
										</style>
										<g class="linegroup">
											`+domstr+`
										</g>
									</svg>`
		}
		
		GRIDMAKER.deleteKnotWithNoConnection=function(){
			let currentknot, currentla, comparela, n, m, dodelete=false, samecount=0;
            let delete_posindexes=[];
			let delete_knotindexes=[];
			let deletecount=0;
			let dmaxloops=99;
			let d=0;
			for(d=0; d<dmaxloops; d++){
				deletecount=0;
				console.log('deletecountloop: ');
				delete_posindexes=[];
				delete_knotindexes=[];
				for(let posindex in posindex2la){
					currentknot=posindex2la[posindex];
					// do delete 
					if(currentknot.las.length===1){
						currentla=currentknot.las[0];
						// save index for line deletion
						delete_posindexes.push(currentla[2]+'_'+currentla[3]);
						// delet whole knot
						delete_knotindexes.push(posindex);
						console.log('dlete knot: ', posindex, 'endpoint:', currentla[2]+'_'+currentla[3]);
						deletecount++;
					}
				}
				if(deletecount===0){
					break;
				}
				// delet knots
				for(const posindex of delete_knotindexes){
					delete posindex2la[posindex];
					console.log('DELTED:', posindex);
				}
				// remove lines with no connections
				n=delete_posindexes.length;
				while(n--){
					if(posindex2la.hasOwnProperty(delete_posindexes[n])){
						currentknot=posindex2la[delete_posindexes[n]];
						m=currentknot.las.length;
						while(m--){
							// if endpoint is deleted
							currentla=currentknot.las[m];
							if(!posindex2la.hasOwnProperty(currentla[2]+'_'+currentla[3])){
								currentknot.las.splice(m,1);
							}
						}
					}

				}
			}
		}
		// ADD GRID
		GRIDMAKER.addGrid=function(svgel){
			const el=asEl(`<div class="grideldraghelper" cursor-drag cursor-dragging touch-able="move rotate scale scalegrabber rotategrabber deletebutton setstyletransformvars" draggable="false"  ondragstart="return false;">
							`+svgel.outerHTML+`
						</div>`);
			grids_layer.appendChild(el);
			makeAllTouchAble();
			const stagetransobj=getTransObj(stage);
			const rect=el.getBoundingClientRect();
			const scale=1.5;
			const stagerect=stage.getBoundingClientRect();
			const insertx=-(el.offsetWidth*scale-stagerect.width)/2+stage.offsetWidth/2;//(-stagetransobj.gx+stageparentrect.width/2-(rect.width/2)*scale)/stagetransobj.gscx;
			const inserty=-(el.offsetHeight*scale-stagerect.height)/2+stage.offsetHeight/2;;//(-stagetransobj.gy+stageparentrect.height/2-(rect.height/2)*scale)/stagetransobj.gscy;
			setTransform(el, {x:insertx, y:inserty, scx:scale, scy:scale, rot:0});
			touchables_mouseDown_touchStart_hand({target:el});
			DRAG_END();
		}
		// Helper
		function xyHitsPolygonpoints(px, py, polygonpoints) {
			const v=polygonpoints.split(' ').join(',').split(',');
			console.log(v);
			let hits=0;
			let p1x, p1y, p2x, p2y;
			let l=v.length, i;
			console.log('px', px, 'py', py);
			for(i=0; i<l-2; i+=2){
				p1x=parseFloat(v[i]);
				p1y=parseFloat(v[i+1]);
				p2x=parseFloat(v[i+2]);
				p2y=parseFloat(v[i+3]);
				if(((p1y>py)!==(p2y>py)) && px<(p2x-p1x)*(py-p1y)/(p2y-p1y)+p1x){
					hits++;
				}				
			}
			if(hits&1){
				return true;
			}else{
				return false;
			}
		}
		function getAngle(cx, cy, x1, y1, x2 ,y2){
			let angle=Math.atan2(y2-cy, x2-cx)-Math.atan2(y1-cy, x1-cx);
			if(angle<0){
				angle+=2*Math.PI; 
			}
			return angle;
        }
		function getIntersectingPoint(x1, y1, x2, y2, x3, y3, x4, y4) {
			//  lines have no length
			if((x1===x2 && y1===y2) || (x3===x4 && y3===y4)) {
				return null;
			}
			const denominator =((y4-y3)*(x2-x1)-(x4-x3)*(y2-y1))
		  	// lines are parallel
			if(denominator===0) {
				return null;
			}
			const ua=((x4-x3)*(y1-y3)-(y4-y3)*(x1-x3))/denominator;
			const ub=((x2-x1)*(y1-y3)-(y2-y1)*(x1-x3))/denominator;
			// intersetion occurs not on lines
			if(ua<=0 || ua>=1 || ub<=0 || ub>=1){
				return null;
			}
			// return intersecting point
			return [x1+ua*(x2-x1), y1+ua*(y2-y1)];
		}
		// nearesst point
		function isLeftToLine(px, py, linearr) {
    		const ax=linearr[0];
			const ay=linearr[1];
			const bx=linearr[2];
			const by=linearr[3];
			const ab_dx=bx-ax;
			const ab_dy=by-ay;
			const ap_dx=px-ax;
			const ap_dy=py-ay;
			return (ab_dx*ap_dy-ab_dy*ap_dx)<1;
		}
		function getNearestPointToLinearr(px, py, linearr) {
    		const ax=linearr[0];
			const ay=linearr[1];
			const bx=linearr[2];
			const by=linearr[3];
			const ab_dx=bx-ax;
			const ab_dy=by-ay;
			const ap_dx=px-ax;
			const ap_dy=py-ay;
			const dist2=ab_dx*ab_dx+ab_dy*ab_dy;
			const dot=ap_dx*ab_dx+ap_dy*ab_dy;
			const t=Math.min(1, Math.max(0, dot/dist2));
			return [ax+ab_dx*t, ay+ab_dy*t];
		}
		function toGrid(n){
			return Math.round(n);
			// not on threshold
			if(Math.round(n+0.3)===Math.round(n)){
				return  Math.round(n);
			// on threshhold
			}else{
				if(Math.round(n+0.15)===Math.round(n)){
					return  Math.round(n);
				}else{
					return  Math.round(n+0.15); 
				}
			}
		}
		
		function getNearestLinearrIndex(px, py, linearrs){
			ui_layer.innerHTML='';
			//
			let la, nla=null, a_min=-1, dist2_min=Number.MAX_VALUE;
			// get nearest line
			for(let a=0; a<linearrs.length; a++){
				la=linearrs[a];
				const np=getNearestPointToLinearr(px, py, la);
				const dx=np[0]-px;
				const dy=np[1]-py;
				const dist2=dx*dx+dy*dy;
				if(dist2<dist2_min){
					a_min=a;
					dist2_min=dist2;
				}
			}
			return a_min;
		}
		function transformLinearrs(linearrs, transformel){
			// transformel
			// get nearest line
			let t=getTransform(transformel);
			for(let a=0; a<linearrs.length; a++){
				la=linearrs[a];
				const p1={x:la[0], y:la[1] , z:0}; 
				const p2={x:la[2], y:la[3] , z:0}; 
				scaleVector(p1, t.scale);
				rotateVector(p1, t.rot);
				addVector(p1, t.pos);
				scaleVector(p2, t.scale);
				rotateVector(p2, t.rot);
				addVector(p2, t.pos);
				la[0]=p1.x;
				la[1]=p1.y;
				la[2]=p2.x;
				la[3]=p2.y;
			}
		}
		
		function getBrokenLinearrs(ls){
			let a=0, b=0, c=0;
			// to grid
			a=ls.length;
			while(a--){
				ls[a][0]=toGrid(ls[a][0]);
				ls[a][1]=toGrid(ls[a][1]);
				ls[a][2]=toGrid(ls[a][2]);
				ls[a][3]=toGrid(ls[a][3]);
			}
			//let ls=lines2Linearrs(lines);
			for(let a=0; a<ls.length; a++){
				for(let b=0; b<ls.length; b++){
					const ip=getIntersectingPoint(ls[a][0], ls[a][1], ls[a][2], ls[a][3], ls[b][0], ls[b][1], ls[b][2], ls[b][3]);
					if(ip!==null){
						const px=toGrid(ip[0]);
						const py=toGrid(ip[1]);
						
						// add new ones
						ls.push([ls[a][2], ls[a][3], px, py]);
						ls.push([ls[b][2], ls[b][3], px, py]);
						
						// shorten to intersection point
						ls[a][2]=px;
						ls[a][3]=py;
						ls[b][2]=px;
						ls[b][3]=py;
					}
				}
			}
			// remove null line and toGrid
			a=ls.length;
			while(a--){
				if(ls[a][0]===ls[a][2] && ls[a][1]===ls[a][3]){
					ls.splice(a, 1);
				}
			}
			console.log('getBrokenLinearrs ls:',ls);
			return ls;
		}
		
		function lines2Linearrs(lines){
			const linearrs=[]; 
			let n=lines.length;
			while(n--){
				linearrs.push(line2Linearr(lines[n]));
			}
			return linearrs;
		}
		function line2Linearr(line){
			return [attr2Int(line, 'x1'), attr2Int(line, 'y1'), attr2Int(line, 'x2'), attr2Int(line, 'y2')];
			return [attr2Int(line, 'x1'), attr2Int(line, 'y1'), attr2Int(line, 'x2'), attr2Int(line, 'y2')];
		}
		function linearrs2Linesdomstr(linearrs){
			let domstr='';
			let n=linearrs.length;
			while(n--){
				domstr+=linearr2Linedomstr(linearrs[n]);
			}
			return domstr;
		}
		function linearr2Linedomstr(linearr){
			if(isNaN(linearr[0]) || isNaN(linearr[1]) || isNaN(linearr[2]) || isNaN(linearr[3])){
				return '';
			}
			return '<line x1="'+linearr[0]+'" y1="'+linearr[1]+'" x2="'+linearr[2]+'" y2="'+linearr[3]+'" />';
		}
		function attr2Int(el, attr){
			//**** ATTENTION!"!!!!!*** //// JUST A TEST!!!!
			return parseFloat(el.getAttribute(attr));
			/*
			if(el.hasAttribute(attr)){
				return Math.round(el.getAttribute(attr));
			}else{
				return 0;
			}
			*/
		}
		
		GRIDMAKER.openHallOfFameForm=function(){
			GRIDMAKER.stageToCanvas(
				function(){
					gridmaker.setAttribute('halloffameform-is-open', '');
				}
			);
			
		}
		GRIDMAKER.closeHallOfFameForm=function(){
			BLUEPRINTS['resetInputGroup'](document.querySelectorAll('*[inputgroup-name="halloffameform"]')[0]);
			gridmaker.removeAttribute('halloffameform-is-open');
			gridmaker.removeAttribute('is-sending-data');
		}
	
		
		GRIDMAKER.initStage=function(){
			const svgel_crop=asEl(`<div class="grideldraghelper" style="display:none;" >
										<svg class="gridel" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" viewBox="0 0 2880 2880" width="2880" height="2880">
											<g class="linegroup">
												<line x1="0" y1="0" x2="2880" y2="0"/>
												<line x1="2880" y1="0" x2="2880" y2="2880"/>
												<line x1="2880" y1="2880" x2="0" y2="2880"/>
												<line x1="0" y1="2880" x2="0" y2="0"/>
											</g>
										</svg>
									</div>`);
			grids_layer.appendChild(svgel_crop);
			GRIDMAKER.setToolMode('placegrids');
		}
		// SET TOOL MODE
		GRIDMAKER.setToolMode=function(mode){ 
			document.body.setAttribute('current-tool-mode', mode);
		}
		
		//
		GRIDMAKER.savebt_onclick=function(event){
			GRIDMAKER.stageToCanvas(
				function(){
					const filename='grid-maker';
					if(window.navigator.msSaveBlob){
						window.navigator.msSaveBlob(canvas.msToBlob, filename+'.png');
					}else{
						const a=document.createElement('A');
						document.body.appendChild(a);
						a.href=canvas.toDataURL('image/jpeg');
						a.download=filename+'.jpg';
						a.click();
						a.parentElement.removeChild(a);
					}
				}
			);
		}
		
		GRIDMAKER.plublishtohalloffamebt_onclick=function(e){
			let groupel=e.currentTarget.closest('*[inputgroup-name="halloffameform"]');
			if(groupel!==null){
				const groupobj=BLUEPRINTS['inputGroupToObj'](groupel);
				const inputs=groupobj['inputs'];
				console.log(groupobj);
				let callobj={};
				let parobj={};
				gridmaker.setAttribute('is-sending-data', '');
				parobj['imgdataurl']=canvas.toDataURL('image/jpeg');
				parobj['infoobj']={};
				parobj['infoobj']['toolslug']='gridmaker';
				parobj['infoobj']['username']=inputs['username']['value'];
				parobj['infoobj']['artworktitle']=inputs['artworktitle']['value'];
				parobj['infoobj']['city']=inputs['city']['value'];
				parobj['infoobj']['einwilligung']=inputs['einwilligung']['value'];
				callobj['api-func-component']='GRIDMAKER';
				callobj['api-func-name']='saveToHallOfFame';
				callobj['api-target-id']='body';
				callobj['api-insert-cmd']='beforeend';
				callobj['api-func-parobj']=parobj;
				API_CALL([callobj]);
			}
			
		}
		
		GRIDMAKER.stageToCanvas=function(oncompleted_func){
			console.log('GRIDMAKER.stageToCanvas');
			const compstyle=window.getComputedStyle(stage, null);
			const w=parseFloat(compstyle.getPropertyValue('width'));
			const h=parseFloat(compstyle.getPropertyValue('height'));
			const brokengrid_svg=stage.querySelectorAll('svg.brokengrid')[0];
			const svgs=[fills_svg, brokengrid_svg];
			let svgrendercount=0;
			canvas.width=w;
			canvas.height=h; 
			context.clearRect(0, 0, canvas.width, canvas.height);
			context.fillStyle='#000000';
			context.fillRect(0, 0, canvas.width, canvas.height);
			function renderSVG(_svg){
				console.log('renderSVG',_svg)
				const img=new Image();				
				const cleansvg=asEl(_svg.outerHTML);
				img.onload=function() {
					context.drawImage(img, 0, 0);
					svgrendercount++
					if(svgrendercount===svgs.length){
						oncompleted_func();
					}else{
						renderSVG(svgs[svgrendercount]);
					}
				}
				let xmlstr='';
				try {
					xmlstr=new XMLSerializer().serializeToString(cleansvg);
				}catch(e){
					console.log(e);
				}
				img.src='data:image/svg+xml;base64,'+btoa(unescape(encodeURIComponent(xmlstr)));
			}
			renderSVG(svgs[svgrendercount]);
		}
		GRIDMAKER.INIT();
	}
	



})();


