<?php
/**
* @license
* Copyright 2021-2022 Martin Härtlein
* SPDX-License-Identifier: MIT
**/
/**
The MIT License

Copyright © 2021-2022 Martin Härtlein

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
**/

// FORCE HTTPS
if($_SERVER['HTTPS']!='on'){
    header('Location: https://'.$_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI']);
    exit();
}
// ERROR REPORTING
$INITIAL_ERROR_REPORTING=-1;
error_reporting($INITIAL_ERROR_REPORTING);
ini_set('display_errors', 'On');

// SESSION
if(session_status()==PHP_SESSION_NONE){
    session_start();
}
// GLOBAL CONSTANTS
define('DR', $_SERVER['DOCUMENT_ROOT']);
define('BASEDIR', ltrim(str_replace(DR, '', dirname($_SERVER['SCRIPT_FILENAME']).'/'), '/'));
define('ROOTPATH', 'https://'.rtrim($_SERVER['HTTP_HOST'].'/'.BASEDIR, '/').'/');
define('ROOT_MEDIA_PATH', 'data/media/');
define('ROOT_DB_PATH', 'data/db/');
define('ROOT_PAGES', 'data/pages/');
define('LOGIN_SESSION_ID', 'SID0123456789');
define('SECRET_KEY', 'SK0123456789');
$_SESSION['SECRET_KEY']=SECRET_KEY;

// GLOBAL VARS
$DOMSTR_TITLE='*PROTOTYPE* Studio Digital';
$DOMSTR_BODY='';
$DOMSTR_CONSOLE='';
$DOMSTR_SCRIPTTAGS='';
$DOMSTR_METATAGS='';
$DOMSTR_BODY_ATTRIBUTES='';
$DOMSTR_HEAD_CODESTR='';

// URL SHEME - $_GET['params'] see .htaccess RewriteRule / define URLPARTS and HTMLRELPATH 
$HTMLRELPATH='';
$URLPARTS=[];
if(isset($_GET['params'])){
	$URLPARTS=explode('/', trim($_GET['params'], '/'));
	// calculate relative path ($HTMLRELPATH) for html src urls
	$slashcount=substr_count($_GET['params'], '/');
	for($n=0; $n<$slashcount ;$n++){
		$HTMLRELPATH.='../';
	}
}
define('URLPARTS', $URLPARTS);
define('HTMLRELPATH', $HTMLRELPATH);

// REDIRECT TO HOMEPAGE
if(count($URLPARTS)===0){
	header("HTTP/1.1 301 Moved Permanently");
	header('Location: https://'.$_SERVER['HTTP_HOST'].$_SERVER['REQUEST_URI'].'startseite');
	exit;
}
$URLPARTS_LAST=$URLPARTS[count($URLPARTS)-1];
$DOMSTR_BODY_ATTRIBUTES=' url-lastpart="'.$URLPARTS_LAST.'"';
$DOMSTR_BODY_ATTRIBUTES.=' url-firstpart="'.$URLPARTS[0].'"';
$DOMSTR_BODY_ATTRIBUTES.=' target-id="body"';

// IS API 
$ISAPI=false;
if($URLPARTS_LAST==='API'){
	$ISAPI=true;
}

// IS UPLOAD 
$ISUPLOAD=false;
if($URLPARTS_LAST==='UPLOAD'){
	$ISUPLOAD=true;
}

// IS PDF MAKER
$ISPDFMAKER=false;
if($URLPARTS_LAST==='PDF_MAKER'){
	$ISPDFMAKER=true;
}

// CORE LOADER
include_once('core/UTILS.php');
include_once('core/MEDIA.php');
AUTO_INCLUDE_PHP_CSS_JS_byFolder('core/engine/');
AUTO_INCLUDE_PHP_CSS_JS_byFolder('core/main/');
AUTO_INCLUDE_PHP_CSS_JS_byFolder('core/blueprints/');

// PDF MAKER
if($ISPDFMAKER){
	include('core/PDF_MAKER.php');
	exit();
}

// INCLUDE API AND UPLOAD
if($ISAPI || $ISUPLOAD){
	// CONTENT LOADER
	$API_TOKEN='';
	$postobj=json2obj(file_get_contents('php://input'));
	if($postobj!==NULL){
		$API_TOKEN=$postobj['apitoken'];
	}else if($_POST['apitoken']!==NULL){
		$API_TOKEN=$_POST['apitoken'];
	}else{
		exit('missing api token');
	}
	define('API_TOKEN', $API_TOKEN);
	$URLPARTS=$_SESSION['API_TOKENS'][API_TOKEN]['URLPARTS'];
	define('ROOT_CURRENTPAGE', ROOT_PAGES.$URLPARTS[0].'/');
	AUTO_INCLUDE_PHPFUNCTIONS_byFolder(ROOT_CURRENTPAGE);
	// API
	if($ISAPI){
		include('core/API.php');
		exit();
	}
	// UPLOAD
	if($ISUPLOAD){
		include('core/UPLOAD.php');
		exit();
	}
}

// SET API TOKENS SESSIONS VARIABLE FOR USE IN API AND UPLOAD BY "URLID"
$URLID=$_GET['params'];
if(!isset($_SESSION['API_TOKEN_BY_URLID'])){
	$_SESSION['API_TOKEN_BY_URLID']=[];
}
if(!isset($_SESSION['API_TOKENS'])){
	$_SESSION['API_TOKENS']=[];
}
if(!isset($_SESSION['API_TOKEN_BY_URLID'][$URLID])){
	$NEW_APITOKEN=bin2hex(random_bytes(16));
	$_SESSION['API_TOKEN_BY_URLID'][$URLID]=$NEW_APITOKEN;
	$_SESSION['API_TOKENS'][$NEW_APITOKEN]=[];
	$_SESSION['API_TOKENS'][$NEW_APITOKEN]['URLPARTS']=$URLPARTS;
}
define('API_TOKEN', $_SESSION['API_TOKEN_BY_URLID'][$URLID]);
$DOMSTR_SCRIPTTAGS.='<script>window.API_TOKEN="'.API_TOKEN.'";</script>';

// CONTENT LOADER
// inlcude php js and css files by first urlpart from "data/pages/" (ROOT_PAGE)
$notfound=false;
if(count(URLPARTS)===1){
	define('ROOT_CURRENTPAGE', ROOT_PAGES.URLPARTS[0].'/');
	// hallo of fame viewer
	if(substr(URLPARTS[0], 0, 8)==='artwork_'){
		AUTO_INCLUDE_PHP_CSS_JS_byFolder('data/pages/hall-of-fame-viewer/', true);
	}else{
		if(is_dir(ROOT_CURRENTPAGE)){
			AUTO_INCLUDE_PHP_CSS_JS_byFolder(ROOT_CURRENTPAGE, true);
		}else{
			$notfound=true;
		}
	}
}

// NOT FOUND
if($notfound===true){
	header("HTTP/1.0 404 Not Found");
	header('Location: '.ROOTPATH.'startseite');
	exit;
}
?>
<!doctype html>
<html lang="de">
	<head>
		<meta charset="utf-8">
		<title><?php echo $DOMSTR_TITLE; ?></title>
		<!-- SEARCH ENGINE INDEXING -->
		<meta name="robots" content="noindex" />
		<!-- VIEWPORT -->
		<meta name="viewport" content="width=device-width, user-scalable=no, initial-scale=1, viewport-fit=cover">
		<!-- JS HTMLRELPATH, ROOTPATH -->
		<script>
			const HTMLRELPATH='<?php echo HTMLRELPATH; ?>';
			const ROOTPATH='<?php echo ROOTPATH; ?>';
		</script>
		<!-- FAVICONS -->
		<link rel="apple-touch-icon" sizes="180x180" href="<?php echo HTMLRELPATH; ?>core/assets/favicons/apple-touch-icon.png">
		<link rel="icon" type="image/png" sizes="32x32" href="<?php echo HTMLRELPATH; ?>core/assets/favicons/favicon-32x32.png">
		<link rel="icon" type="image/png" sizes="16x16" href="<?php echo HTMLRELPATH; ?>core/assets/favicons/favicon-16x16.png">
		<link rel="manifest" href="<?php echo HTMLRELPATH; ?>core/assets/favicons/site.webmanifest">
		<link rel="mask-icon" href="<?php echo HTMLRELPATH; ?>core/assets/favicons/safari-pinned-tab.svg" color="#ffffff">
		<link rel="shortcut icon" href="<?php echo HTMLRELPATH; ?>core/assets/favicons/favicon.ico">
		<meta name="msapplication-TileColor" content="#000000">
		<meta name="msapplication-config" content="<?php echo HTMLRELPATH; ?>core/assets/favicons/browserconfig.xml">
		<!-- HEAD CODE STRING -->
		<?php echo $DOMSTR_HEAD_CODESTR; ?>
		<!-- THEME -->
		<meta name="theme-color" content="#000000">
		<!-- META DOMSTR -->
		<?php echo $DOMSTR_METATAGS; ?>
		<!-- ICON STYLE -->
		<?php echo  svgFolder2HTMLStyleTag('core/assets/icons'); ?>
		<!-- STYLES -->
		<?php echo $DOMSTR_STYLETAGS; ?>
		<!-- SCRIPTS -->
		<?php echo $DOMSTR_SCRIPTTAGS; ?>
		<?php 
		if(file_exists('google-analytics.php')){
		  	include_once('google-analytics.php'); 
		}
		?>
	</head>
	<body <?php echo $DOMSTR_BODY_ATTRIBUTES; ?>>
		<?php echo $DOMSTR_BODY; ?>
	</body>
</html>

<?php 
// LASTPAGENAME
$_SESSION['LASTPAGENAME']=$URLPARTS[0];


?>
